/* Kompilator konfiguraku na web
 * (c)16.10.1999 by Marek Zelem
 * (c)9.7.2000 by Marek Zelem
 */

#include <mmm/compiler.h>
#include <mmm/lex.h>
#include <mmm/dynamic.h>
#include <stdlib.h>
#include <string.h>
#include "web_compile.h"

struct compiler_preprocessor_class *gcc_preprocessor_create( char *filename );
#pragma weak gcc_preprocessor_create

lextab_t operators[]={
	{"{",T|'{',0},	{"}",T|'}',0},
	{"=",T|'=',0},
	{",",T|',',0},	{";",T|';',0},
	{"(",T|';',0},	{")",T|';',0},
	{".",T|'.',0},	{"|",T|'|',0},
	{"//",LS_comment_line,LEX_END},
	{NULL,END,0}
};

lextab_t keywords[]={
//	{"sekcia",Lsekcia,0},
//	{"sub",Lsub,0},
	{"hidden",Lhidden,0},
	{"NULL",Lstr,0},
	{NULL,END,0}
};

/* definicia pravidiel pre lexikalny analyzator */

lextab_t rules_start[]= {
	{" \t\n\r",LS_start,LEX_VOID},
	{"a-zA-Z_@",LS_ident,LEX_CONT},
	{"\"",LS_string,LEX_VOID},
	{"<",LS_file,LEX_VOID},
	{"%",LS_var,LEX_VOID},
	{"#",LS_comment_line,LEX_VOID},
	{NULL,END,0}
};
lextab_t rules_ident[]= {
	{"a-zA-Z_0-9",0,LEX_CONT},
	{"!",LS_start,LEX_END},
	{NULL,END,0}
};
lextab_t rules_string[]={
	{"\"",LS_start,LEX_VOID|LEX_END},
	{"\\",0,LEX_BSLASH|LEX_CONT},
	{"!",0,LEX_CONT},
	{NULL,END,0}
};
lextab_t rules_file[]={
	{">",LS_start,LEX_VOID|LEX_END},
	{"!",0,LEX_CONT},
	{NULL,END,0}
};
lextab_t rules_var[]={
	{"%",LS_start,LEX_VOID|LEX_END},
	{"!",0,LEX_CONT},
	{NULL,END,0}
};
lextab_t rules_comment_line[]={
	{"\n",LS_start,LEX_VOID},
	{"!",0,LEX_VOID},
	{NULL,END,0}
};

void gen_lex_ident( char *buf, int len, sym_t *sym, unsigned long *data, sym_t want );
void gen_lex_string( char *buf, int len, sym_t *sym, unsigned long *data, sym_t want );
void gen_lex_file( char *buf, int len, sym_t *sym, unsigned long *data, sym_t want );
void gen_lex_var( char *buf, int len, sym_t *sym, unsigned long *data, sym_t want );
lexstattab_t lex_tab[]={
	{LS_start,operators,rules_start,NULL,NULL},
	{LS_ident,NULL,rules_ident,keywords,gen_lex_ident},
//	{LS_ident,NULL,rules_ident,NULL,gen_lex_ident},
	{LS_string,NULL,rules_string,NULL,gen_lex_string},
	{LS_file,NULL,rules_file,NULL,gen_lex_file},
	{LS_var,NULL,rules_var,NULL,gen_lex_var},
	{LS_comment_line,NULL,rules_comment_line,NULL,NULL},
	{END}
};

void gen_lex_ident( char *buf, int len, sym_t *sym, unsigned long *data, sym_t want )
{ char *s;
	*sym=Lid;
	if( (s=malloc(strlen(buf)+2))!=NULL )
	{	strcpy(s,buf);
		SET_UNUSED(s);
	}
	*data=(unsigned long)s;
}

void gen_lex_string( char *buf, int len, sym_t *sym, unsigned long *data, sym_t want )
{	*sym=Lstr;
	*data=(unsigned long)strdup(buf);
}

void gen_lex_file( char *buf, int len, sym_t *sym, unsigned long *data, sym_t want )
{	*sym=Lfile;
	*data=(unsigned long)strdup(buf);
}

void gen_lex_var( char *buf, int len, sym_t *sym, unsigned long *data, sym_t want )
{	*data=(unsigned long)strdup(buf);
	*sym=Lvar;
}

/* GRAMATIKA */
struct compile_tab_s gramatika[]={
	{START,{END},{Lid,BLOK1,J,END}},
	{BLOK1,{END},{oSNEW,T|'{',P,oSEND,T|'}',END}},
	{BLOK2,{END},{oSNEW,T|'{',P,oSSTR,T|'}',END}},
	{P,{Lhidden,END},{Lhidden,oHIDB,T|'{',P,oHIDE,T|'}',J,P,END}},
	{P,{Lid,END},{Lid,P1,J,P,END}},
	{P,{END},{END}},
	{P1,{T|'{',END},{BLOK1,END}},
	{P1,{END},{oNAM,T|'=',H,END}},
	{J,{T|';',END},{T|';',END}},
	{J,{END},{END}},
	{H,{T|'.',END},{T|'.',HH,oADDVAL,H1,END}},
	{H,{T|'|',END},{T|'|',HH,oFILTERVAL,H1,END}},
	{H,{END},{HH,oVAL,H1,END}},
	{HH,{Lstr,END},{Lstr,oSTR,END}},
	{HH,{Lvar,END},{Lvar,oCPSTR,END}},
	{HH,{Lfile,END},{Lfile,oFSTR,END}},
	{HH,{T|'{',END},{VAL|0,BLOK2,END}},
	{H1,{T|',',END},{T|',',H,END}},
	{H1,{T|'.',T|'|',END},{H,END}},
	{H1,{Lstr,END},{T|',',H,END}},
	{H1,{END},{END}},
	{END}
};

/* main */
compiler_class_t *compiler;

char *text_filter=NULL;
char *text_exec=NULL;

int main( int argc, char *argv[] )
{ struct compiler_preprocessor_class *prep;
  char buf[128];
  int a,use_gcc=0;
	printf(	"Opos generator suborovej struktury\n"
		"(c)1999 Marek Zelem\n"
		"Ver. 21.10.2000\n"
		);
	lds_verlib(buf);	printf("%s\n",buf);
	compiler_verlib(buf);	printf("%s\n",buf);
	printf("\n");

	a=1;
	while( a<argc )
	{	if( argv[a][0]=='-' )
		{	if( argv[a][1]=='t' && a+1<argc )
			{	text_exec=argv[a+1];	a+=1;	}
			if( argv[a][1]=='f' && a+1<argc )
			{	text_filter=argv[a+1];	a+=1;	}
			else if( argv[a][1]=='g' )
			{	use_gcc=1;	}
			else if( argv[a][1]=='h' )
			{	printf("pouzitie: %s [-g] [-f <prog>] [-t <prog>] <conf.subor>\n"
					,argv[0]);
				printf(	"  -g - pouzit \"gcc -E\" ako preprocesor\n"
					"  -f - vsetky retazce filtruj filtrom prog\n"
					"  -t - pri generovani textovej ver. spusti prog namiesto kopirovnia\n"
					"  conf.subor - konfiguracny subor\n");
				exit(0);
			}
			else
				fprintf(stderr,"Nespravny prepinac %s\n",
						argv[a]);
		}
		else break;
		a++;
	}
	if( a!=argc-1 )
	{	fprintf(stderr,"Nespravny pocet argumentov\n");
		exit(0);
	}
	
	if( (compiler=compiler_create())==NULL )
		return(-1);
	compiler->add_table(compiler,gramatika);
	if( use_gcc )
	{	if( gcc_preprocessor_create!=NULL )
			prep=gcc_preprocessor_create(argv[a]);
		else
		{	fprintf(stderr,"gcc_preprocessor nieje zakompilovany\n");
			exit(0);
		}
	}
	else		prep=nul_preprocessor_create(argv[a]);
	compiler->lex=lex_create(lex_tab,prep);
	inc_use(prep);
	compiler->err=&err_tab;
	compiler->out=out_create(compiler);
	compiler->compile(compiler,START);
	dec_use(prep);
	printf("Errors: %d\nWarnings: %d\n",compiler->err->errors,compiler->err->warnings);
	compiler->destroy(compiler);
	return(0);
}

